//#region GPL License.
//* 
// * SourceGear Vault Integration Tasks for NAnt
// * Copyright (C) 2003  Brian M. Schkerke
//
// * This program is free software; you can redistribute it and/or modify
// * it under the terms of the GNU General Public License as published by
// * the Free Software Foundation; either version 2 of the License, or
// * (at your option) any later version.
//
// * This program is distributed in the hope that it will be useful,
// * but WITHOUT ANY WARRANTY; without even the implied warranty of
// * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// * GNU General Public License for more details.
//
// * You should have received a copy of the GNU General Public License
// * along with this program; if not, write to the Free Software
// * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
// */
//
//#endregion
//
// This task developed by Perry Ismangil
//

using System;
using System.IO;
using System.Text;
using NAnt.Core;
using NAnt.Core.Types;
using NAnt.Core.Attributes;
using NAnt.Core.Tasks;
using VaultClientOperationsLib;
using VaultClientNetLib;
using VaultClientNetLib.ClientService;
using VaultLib;


namespace NAnt.Contrib.Tasks.SourceGearVault
{
	/// <summary>
	/// Delete an existing label from a repository folder.
	/// <seealso cref="vaultlabel"/>
	/// </summary>
	/// <example>
	/// <code>
	/// <![CDATA[<vaultdeletelabel url="http://localhost" 
	///     username="username" 
	///     password="password" 
	///     repository="repository" 
	///     labelstring="Build ${build.revision}"
	///     path="$/Project/Directory" />
	///   ]]></code>
	/// </example>
	[TaskName("vaultdeletelabel")]
	public class vaultdeletelabel : VaultBase
	{
		#region Private Variable Declarations.
		private string				pRepository				= "";
		private string				pRepositoryPath			= "";
		private string				pLabelString			= "";
		private bool					pFailOnMissing			= false;
		#endregion
	
		#region Accessor Declarations.
		/// <summary>
		/// The Vault repository to be used.
		/// </summary>
		[TaskAttribute( "repository", Required = true )]
		public string repository
		{ 
			get { return pRepository;						}
			set { pRepository = value;						}
		}
		//
		/// <summary>
		/// The path to the folder which will be labeled
		/// </summary>
		[TaskAttribute( "path", Required = true )]
		public string path
		{
			get { return pRepositoryPath;					}
			set { pRepositoryPath = value;					}
		}
		//
		/// <summary>
		/// The string which be used as the label text.
		/// </summary>
		/// <example>labelstring="Build ${build.major}.${build.minor}.${build.build}.${build.revision}</example>
		[TaskAttribute( "labelstring", Required = true )]
		public string labelstring
		{
			get { return pLabelString;						}
			set { pLabelString = value;						}
		}

		/// <summary>
		/// Determines whether the delete label task should fail or not if the label is missing. 
		/// By default, the task will not fail.
		/// </summary>
		[TaskAttribute( "failonmissing", Required = false )]
		public bool failonmissing
		{
			get { return this.pFailOnMissing;						}
			set { this.pFailOnMissing = value;						}
		}
		#endregion
	
		#region ExecuteTask.
		/// <summary>
		/// Executes the task at hand.
		/// </summary>
		/// <exception>BuildException</exception>
		protected override void ExecuteTask() 
		{
			AccessLevel = VaultConnection.AccessLevelType.Client;
			Login();
			//
			try 
			{
				SelectRepository( repository );
				DeleteLabel( pRepositoryPath, pLabelString, pFailOnMissing );	
			}
			finally
			{
				Logout();
			}
		}
		#endregion
	
		#region Support functions.
		/// <summary>
		/// Delete a label from the specified path
		/// </summary>
		/// <exception>BuildException</exception>
		private void DeleteLabel( string path, string labelString, bool failOnMissing )
		{
			long labelID = 0;
			long rootID = 0;
			string[] discoveredPaths;
			VaultClientTreeObject labelStructure=null;

			string normalizedPath = VaultLib.RepositoryPath.NormalizeFolder( path );
			//
			//if( IsVaultFolder( normalizedPath ) == true )
			//{
			VaultClientTreeObject vaultClientObject = ClientInstance.TreeCache.Repository.Root.FindTreeObjectRecursive( normalizedPath );


			if (vaultClientObject != null)
			{
				string msgNotFound = string.Format("Could not find label \"{0}\" created at item \"{1}\".  ", labelString, normalizedPath);

				try
				{
					// Nicked from CLC:
					// There isn't a good API to get a label ID based on a label name, so just get the whole structure
					//_ci.GetByLabel_GetStructure(strReposPath, strLabelName, ref labelID, "", out discoveredPaths, out labelStructure, out rootID);
					ClientInstance.GetByLabel_GetStructure(normalizedPath, labelString, ref labelID, "", out discoveredPaths, out labelStructure, out rootID);
			
					if (vaultClientObject.ID == rootID && labelID != 0)
					{
						int ret  = ClientInstance.DeleteLabel(normalizedPath, labelID);
					
						if( ret != VaultStatusCode.Success )
						{
							throw new BuildException("The label " + labelString + " could not be deleted from the Folder " + normalizedPath + " " + VaultClientNetLib.VaultConnection.GetSoapExceptionMessage(ret));
						}
						else
						{
							VaultLog( "Label " + labelString + " successfully deleted from " + vaultClientObject.Name + "." );
						}
					} 
					else 
					{
						if (failOnMissing)
						{
							throw new BuildException(msgNotFound);
						}
						else
						{
							VaultLog(msgNotFound);
						}
					}
				}
				catch (Exception)
				{
					if (failOnMissing)
					{
						throw new BuildException(msgNotFound);
					}
					else
					{
						VaultLog(msgNotFound);
					}
				}
			}
			else
			{
				throw new BuildException( "Folder " + normalizedPath + " not found.", Location );
			}
		}
		#endregion
	}
}
